import pandas as pd

class DataSanatizer:
    """
    For data sanitization utility methods
    """

    @staticmethod
    def _clean_nan_column(dataframe: pd.DataFrame) -> pd.DataFrame:
        """
        The last column of the DataFrame is usually an unamed column with all null values. 
        It should be discared.
        """
        if dataframe.columns[-1].startswith("Unnamed") and dataframe[dataframe.columns[-1]].isna().all():
            return dataframe.iloc[:, :-1]


from cricinfo.helpers.request_helper import RequestHelper

class TestRequestHelper:

    def test_request_header_map(self):
        header_map = RequestHelper.HEADER_MAP()
        assert header_map is not None
        assert len(header_map) >= 1
        assert 'User-Agent' in header_map

    def test_constants(self):
        constants = [
            "PAGE_PARAMETER",
            "TEAM_PARAMETER",
            "TEMPLATE_PARAMETER",
            "TEMPLATE_VALUE",
            "TYPE_PARAMETER",
            "MATCH_FORMAT_PARAMETER",
            "USER_AGENT_PARAMEATER",
            "USER_AGENT_VALUE",
            "REQUEST_URL",
        ]
        for constant in constants:
            assert constant in RequestHelper.__members__


from cricinfo import Team

class TestTeam:

    def test_teams(self):
        teams = [
            "England",
            "Australia",
            "SouthAfrica",
            "WestIndies",
            "NewZealand",
            "India",
            "Pakistan",
            "SriLanka",
            "Zimbawe"
        ]
        for team in teams:
            assert team in Team.__members__


from enum import Enum

class Team(Enum):
    """
    Statistics can be filtered by team. 
    
    For any method where this class is a parameter, it can be passed as :obj:`None`,
    bypassing team filtering.
    """
    England = '1'
    Australia = '2'
    SouthAfrica = '3'
    WestIndies = '4'
    NewZealand = '5'
    India = '6'
    Pakistan = '7'
    SriLanka = '8'
    Zimbawe = '9'


from cricinfo import Cricinfo
from cricinfo import Team
from cricinfo import MatchFormat

class TestCricInfo:

    def setup_method(self):
        self.team = Team.Pakistan
        self.match_format = MatchFormat.T20I

    def test_retrieve_batting_stats(self):
        df = Cricinfo.retrieve_batting_stats(team=self.team, match_format=self.match_format)
        assert df is not None, "Expected dataframe returned from Cricinfo service to not be none."
        assert df.shape[1] == 15, "Expected dataframe for batting stats to have to have 15 columns."
        assert df.shape[0] > 100, "Expected atleast 100 records returned for sample batting stat retrieval."

    def test_retrieve_bowling_stats(self):
        df = Cricinfo.retrieve_bowling_stats(team=self.team, match_format=self.match_format)
        assert df is not None, "Expected dataframe returned from Cricinfo service to not be none."
        assert df.shape[1] == 14, "Expected dataframe for bowling stats to have to have 14 columns."
        assert df.shape[0] > 100, "Expected atleast 100 records returned for sample bowling stat retrieval."

    def test_retrive_fielding_stats(self):
        df = Cricinfo.retrieve_fielding_stats(team=self.team, match_format=self.match_format)
        assert df is not None, "Expected dataframe returned from Cricinfo service to not be none."
        assert df.shape[1] == 11, "Expected dataframe for fielding stats to have to have 11 columns."
        assert df.shape[0] > 100, "Expected atleast 100 records returned for sample fielding stat retrieval."
    
    def test_retrive_allround_stats(self):
        df = Cricinfo.retrieve_allround_stats(team=self.team, match_format=self.match_format)
        assert df is not None, "Expected dataframe returned from Cricinfo service to not be none."
        assert df.shape[1] == 14, "Expected dataframe for all-round stats to have to have 14 columns."
        assert df.shape[0] > 100, "Expected atleast 100 records returned for sample all-round stat retrieval."

    def test_retrieve_partnership_stats(self):
        df = Cricinfo.retrieve_partnership_stats(team=self.team, match_format=self.match_format)
        assert df is not None, "Expected dataframe returned from Cricinfo service to not be none."
        assert df.shape[1] == 9, "Expected dataframe for partnership stats to have to have 9 columns."
        assert df.shape[0] > 100, "Expected atleast 100 records returned for sample partnership stat retrieval." 

    def test_retrieve_team_stats(self):
        df = Cricinfo.retrieve_team_stats(team=self.team, match_format=self.match_format)
        assert df is not None, "Expected dataframe returned from Cricinfo service to not be none."
        assert df.shape[1] == 13, "Expected dataframe for team stats to have to have 13 columns."
        assert df.shape[0] == 1, "Expected 1 records returned for sample team stat retrieval."

    def test_retrieve_aggregate_stats(self):
        df = Cricinfo.retrieve_aggregate_stats(team=self.team, match_format=self.match_format)
        assert df is not None, "Expected dataframe returned from Cricinfo service to not be none."
        assert df.shape[1] == 10, "Expected dataframe for aggregate team stats to have to have 10 columns."
        assert df.shape[0] == 1, "Expected 1 records returned for sample team stat retrieval."     


from enum import Enum

class StatType(Enum):
    BATTING = "batting"
    BOWLING = "bowling"
    FIELDING = "fielding"
    ALLROUND = "allround"
    PARTNERSHIP = "fow"
    TEAM = "team"
    AGGREGATE = "aggregate"


from cricinfo.helpers.data_sanitizer import DataSanatizer
import pandas as pd
import numpy as np

class TestDataSanitizer:

    def test_clean_nan_column(self):
        df = pd.DataFrame({'number': [1,2,3]})
        df["Unnamed"] = np.nan #create a column with NaN data to mimic cricinfo DF
        cleaned_df = DataSanatizer._clean_nan_column(df)
        assert cleaned_df is not None
        assert not cleaned_df[cleaned_df.columns[-1]].isna().all()


import sys
sys.path.append("...")
from cricinfo.helpers.stat_type import StatType
from cricinfo.services.cricinfo_service import CricinfoService
from cricinfo.team import Team
from cricinfo.match_format import MatchFormat
import pytest

class TestCricinfoService:

    def setup_method(self):
        self.team = Team.Pakistan
        self.match_format = MatchFormat.Test

    def test_retrieve_stats(self):
        df = CricinfoService.retrieve_stats(team=self.team, match_format=self.match_format, stats_type=StatType.TEAM)
        assert df is not None, "Expected dataframe returned from Cricinfo service to not be none."
        assert df.shape[1] == 13, "Expected dataframe for team stats to have to have 13 columns."
        assert df.shape[0] == 1, "Expected 1 records returned for sample team stat retrieval."

    def test_retrieve_stats_without_team_filter(self):
        df = CricinfoService.retrieve_stats(team=None, match_format=self.match_format, stats_type=StatType.TEAM)
        assert df is not None, "Expected dataframe returned from Cricinfo service to not be none."
        assert df.shape[1] == 13, "Expected dataframe for team stats to have to have 13 columns."
        assert df.shape[0] >= 13, "Expected 13 records returned for sample team stat retrieval without team filter."

    def test_invalid_team(self):
        stat_type = StatType.BOWLING
        with pytest.raises(TypeError, match="Invalid type for team"):
            CricinfoService.retrieve_stats(team="INVALID_TEAM", match_format=self.match_format, stats_type=stat_type)

    def test_invalid_format(self):
        stat_type = StatType.BOWLING
        with pytest.raises(TypeError, match="Invalid type for match_format"):
            CricinfoService.retrieve_stats(team=self.team, match_format="INVALID_FORMAT", stats_type=stat_type)

    def test_invalid_stats_type(self):
        with pytest.raises(TypeError, match="Invalid type for stats_type"):
            CricinfoService.retrieve_stats(team=self.team, match_format=self.match_format, stats_type="1")


from setuptools import setup, find_packages

setup(
    name="cricinfo-stats",
    version="1.0.2",
    author="Ali Raza",
    author_email="aaraza1995@gmail.com",
    description="Python/Pandas Client for https://www.espncricinfo.com stats",
    long_description=open("README.md").read(),
    long_description_content_type="text/markdown",
    url="https://github.com/aaraza/cricinfo",
    packages=find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires=">=3.7",  # Adjust as needed
)